﻿Module Globals
    Public CurMode As Integer = 0
    ' Current mode for mytimer background
    '   0 = do nothing ie. Single sweep etc.
    '   1 = Continuous sweep ie. trigger new sweep per tick
    '   2 = AD9850 Cal ie. Set DDS to indicated frequency using clock value per tick.
    '   3 = AD8307 Cal free run ie. Continuious running reading value.
    Public AD9851Flg As Boolean = False
    Public AD9850RefClk As Integer
    Public AD9851RefClk As Integer
    ' Default divisor of 10 assumes ad8307 at 25mv per dB and 2.56 volt = 1024 => /10 gives approx dB
    Public AD8307Div(2) As Double
    Public AD8307Off(2) As Double
    Public CurAD8307Port As Integer = 0
    Public VScale As Integer ' Vertical scale in dB of display window
    Public VOffset As Integer ' Vertical offset (dB) of display window
    Public A2DAvg As Integer = 1
    Public MyVersion As String = My.Application.Info.Version.ToString
    Public CommsUp As Boolean = False
    Public InEdit As Boolean = False
    Public SavedCurve(2) As curve
    Public CurrentCurve(2) As curve ' Curve 0 to 2 are analog channels 
    Public NormalizeIt() As Boolean = {False, False, False}
    Public MarkersOn() As Boolean = {False, False, False}
    Public LevelIt() As Boolean = {False, False, False}
    Public FDH As Double ' Frequency Displayed High
    Public FDL As Double ' Frequency Displayed Low
    Public LDH As Double ' Level Displayed High
    Public LDL As Double ' Level Displayed Low
    Public ScreenWidth As Integer
    Public ScreenHeight As Integer
    Public ScreenColor As Color
    Public GraticuleColor As Color
    Public MarkerColor As Color
    Public ShowVSWR As Boolean = False
    Public SportsMode As Boolean = False
    Public Const maxmarkers As Integer = 8
    Public LevelRef As Integer ' channel to use as leveling reference 0,1,2 anything else( eg 99) = leveling off

    Public Function Phase2String(ByVal d_phase As UInt64) As String

        Dim TempUI As UInt64
        Dim TempUI2 As UInt64
        Dim temptext As String
        TempUI = d_phase
        '' Writen to DDS right to left so control byte first in out string ( Phase MSB = control MSB)
        If AD9851Flg Then
            ' Set the 6 times multiplier for the AD9851
            '  Caution if you send this to a AD9850 by mistake it could do permenant damage. 
            temptext = "D 01,"
        Else
            temptext = "D 00,"
        End If
        TempUI2 = TempUI
        TempUI = Int(TempUI2 / Math.Pow(2, 24))
        If TempUI > 127 Then
            temptext = "Required Frequency too high"
        Else
            temptext = temptext & BInt2Hex(TempUI)
            TempUI2 = TempUI2 - TempUI * Math.Pow(2, 24)
            TempUI = Int(TempUI2 / Math.Pow(2, 16))
            temptext = temptext & "," & BInt2Hex(TempUI)
            TempUI2 = TempUI2 - TempUI * Math.Pow(2, 16)
            TempUI = Int(TempUI2 / Math.Pow(2, 8))
            temptext = temptext & "," & BInt2Hex(TempUI)
            TempUI2 = TempUI2 - TempUI * Math.Pow(2, 8)
            TempUI = TempUI2
            temptext = temptext & "," & BInt2Hex(TempUI)
        End If
        Phase2String = temptext

    End Function
    Public Function String2Volt(ByVal MyText As String) As Integer
        'Return the integer V value (of the 4 hex digits) in received string for ports 0,1,2
        Dim Gotone As Boolean = False ' False if nothing found
        Dim MyChar As String
        Dim MyTempChar As String
        Dim Mycnt As Integer
        Dim MyPosn As Integer = 0 ' What nibble are we up to
        Dim MyType As Integer = 0 ' What type of response is it, 0 = none yet, 1 = Voltage, 2 = Frequency, 3 = i2c read
        Dim MyNib(9) As Integer ' array of raw nibbles in
        Dim MyV As Integer ' current V 

        For Mycnt = 1 To Len(MyText)
            MyChar = Mid(MyText, Mycnt, 1)
            Select Case MyChar
                Case "V", "A" ' Its going to be a voltage
                    MyType = 1
                    MyPosn = 0
                Case "0" To "9"
                    If MyPosn < 10 Then
                        MyNib(MyPosn) = Val(MyChar)
                        MyPosn = MyPosn + 1
                    Else ' too much data
                        ' its an error just clean up
                        MyType = 0
                        MyPosn = 0
                    End If
                Case "a" To "f" ' Hex must use lower case
                    If MyPosn < 10 Then
                        MyTempChar = "&H" & MyChar
                        MyNib(MyPosn) = Val(MyTempChar)
                        MyPosn = MyPosn + 1
                    Else ' too much data
                        ' its an error just clean up
                        MyType = 0
                        MyPosn = 0
                    End If
                Case Else
                    'ignore
            End Select
        Next Mycnt

        Select Case MyType
            Case 1 ' its a voltage
                'first 2 nibbles = voltage num, first nibble always 0 second 0 to 2
                'next four nibbles = value
                Select Case MyPosn
                    Case 6 ' we have normal 6 ( 0 to 5) nibbles
                        Select Case MyNib(1)
                            Case 0, 1, 2 ' get the reading whichever port it is.
                                MyV = MyNib(2) * 4096 + MyNib(3) * 256 + MyNib(4) * 16 + MyNib(5) * 1
                                Gotone = True
                            Case Else
                                ' its an error just clean up
                                MyType = 0
                                MyPosn = 0
                        End Select
                    Case Else
                        ' its an error just clean up
                        MyType = 0
                        MyPosn = 0
                End Select
            Case Else
                ' ignore it for now
        End Select
        If Gotone Then
            String2Volt = MyV
        Else
            String2Volt = 0
        End If
    End Function
    Public Sub MakePoints(ByRef mycurve As curve)
        ReDim mycurve.Ps(mycurve.NumPoints - 1)
        Dim myX As Integer
        Dim myY As Integer
        For i = 0 To mycurve.NumPoints - 1
            myX = ScreenHeight - ScreenHeight * (mycurve.Values(i).DL - LDL) / (LDH - LDL)
            myY = ScreenWidth * (mycurve.Values(i).DF - FDL) / (FDH - FDL)
            If myX > ScreenHeight Then myX = ScreenHeight
            If myX < 0 Then myX = 0
            If myY > ScreenWidth Then myY = ScreenWidth
            If myY < 0 Then myY = 0
            mycurve.Ps(i) = New Point(myY, myX)
        Next
    End Sub

    Public Function Freq2Phase(ByVal Freq As Double) As UInt64
        If AD9851Flg Then
            Freq2Phase = (Freq * Math.Pow(2, 32)) / AD9851RefClk
        Else
            Freq2Phase = (Freq * Math.Pow(2, 32)) / AD9850RefClk
        End If

    End Function
    Public Function LevelAndNormalize(ByVal index As Integer, ByVal CurveNo As Integer) As Double

        If NormalizeIt(CurveNo) Then

            If LevelIt(CurveNo) Then
                ' Normalize and Level so use the saved display level for normalizing
                If CurrentCurve(CurveNo).Values(index).DF = SavedCurve(CurveNo).Values(index).DF Then
                    LevelAndNormalize = CurrentCurve(CurveNo).Values(index).RL - SavedCurve(CurveNo).Values(index).DL - CurrentCurve(LevelRef).Values(index).RL + CurrentCurve(CurveNo).MeOFF
                Else
                    'Something is wrong we shouldn't be normalizing
                    NormalizeIt(CurveNo) = False
                    LevelAndNormalize = 999
                End If
            Else
                'Just Normalize so do against saved raw level.
                If CurrentCurve(CurveNo).Values(index).DF = SavedCurve(CurveNo).Values(index).DF Then
                    LevelAndNormalize = CurrentCurve(CurveNo).Values(index).RL - SavedCurve(CurveNo).Values(index).RL
                Else
                    'Something is wrong we shouldn't be normalizing
                    NormalizeIt(CurveNo) = False
                    LevelAndNormalize = 999
                End If

            End If
        Else
            If LevelIt(CurveNo) Then
                'Just Level
                LevelAndNormalize = CurrentCurve(CurveNo).Values(index).RL - CurrentCurve(LevelRef).Values(index).RL + CurrentCurve(CurveNo).MeOFF
            Else
                'Do Neither
                LevelAndNormalize = CurrentCurve(CurveNo).Values(index).RL
            End If
        End If

    End Function
  
    Public Function SetResetmarker(ByVal index As Integer, ByVal CurveNo As Integer) As Integer
        ' returns either the marker number if a set ( 0-maxmarkers) , -2 if a reset, -1 if no spare markers 
        Dim resetflag As Boolean = False
        Dim TSRM As Integer = -1
        For i = 0 To CurrentCurve(CurveNo).NumMarkers - 1
            If CurrentCurve(CurveNo).Markers(i) = index Then
                'it exists so remove it it
                'Shuffle down other markers
                For j = i To CurrentCurve(CurveNo).NumMarkers - 2
                    CurrentCurve(CurveNo).Markers(j) = CurrentCurve(CurveNo).Markers(j + 1)
                Next
                'make the current last marker = 0
                CurrentCurve(CurveNo).Markers(CurrentCurve(CurveNo).NumMarkers - 1) = 0
                'decrease the number of Markers
                CurrentCurve(CurveNo).NumMarkers = CurrentCurve(CurveNo).NumMarkers - 1
                'Make it a reset
                resetflag = True
                TSRM = -2
                Exit For
            End If
        Next
        If Not resetflag Then
            ' it must be a set
            If CurrentCurve(CurveNo).NumMarkers < maxmarkers Then
                'there is room
                CurrentCurve(CurveNo).Markers(CurrentCurve(CurveNo).NumMarkers) = index
                TSRM = CurrentCurve(CurveNo).NumMarkers
                'increase the number of Markers
                CurrentCurve(CurveNo).NumMarkers = CurrentCurve(CurveNo).NumMarkers + 1

            Else
                'no room

                TSRM = -1
            End If
        End If

        Return TSRM
    End Function
    Public Function Setmarker(ByVal index As Integer, ByVal CurveNo As Integer) As Integer
        ' returns  the marker number if successfull or allready exists( 0-maxmarkers) , or -1 if no spare markers 

        Dim TSRM As Integer = -1
        With CurrentCurve(CurveNo)
            For i = 0 To .NumMarkers - 1
                If .Markers(i) = index Then
                    'it exists so leave it there and remember the marker number
                    TSRM = i
                    Exit For
                End If
            Next
            If TSRM = -1 Then
                ' Try and add it
                If .NumMarkers < maxmarkers Then
                    'there is room
                    .Markers(.NumMarkers) = index
                    TSRM = .NumMarkers
                    'increase the number of Markers
                    .NumMarkers = .NumMarkers + 1
                End If
            End If
        End With

        Return TSRM
    End Function
    Public Function Index2Marker(ByRef TestIndex As Integer, ByRef TestCurveNo As Integer) As Integer
        'for curve TestCurveNo check if we have a marker set at TestIndex and return Marker number (1-Maxmarkers) or -1 if not found.
        Dim FoundMarker As Integer = -1
        For i = 0 To CurrentCurve(TestCurveNo).NumMarkers - 1
            If CurrentCurve(TestCurveNo).Markers(i) = TestIndex Then
                FoundMarker = i + 1
                Exit For
            End If
        Next
        Index2Marker = FoundMarker
    End Function
    Public Sub updatemarkers()
        Markers.MarkList.BeginUpdate()
        Markers.MarkList.Items.Clear()
        For j = 0 To 2
            If MarkersOn(j) Then
                With CurrentCurve(j)

                    For i = 0 To .NumMarkers - 1
                        Markers.MarkList.Items.Add(j.ToString("0_") & (i + 1).ToString("0") & ControlChars.Tab & .Values(.Markers(i)).DF.ToString("###,###,##0") & ControlChars.Tab & .Values(.Markers(i)).DL.ToString("##0.0"))
                    Next

                End With
            End If
        Next
        Markers.MarkList.EndUpdate()
    End Sub
    Public Function Byte2Bin(ByVal DecNum As Byte) As String
        'converts a byte into a string of binary digits
        Dim mystemp As String
        mystemp = "00000000" & Convert.ToString(DecNum, 2)
        mystemp = Microsoft.VisualBasic.Right(mystemp, 8)

        Return mystemp
    End Function
    Public Function BInt2Hex(ByVal DecNum As Integer) As String
        'converts a byte stored in an int into a two char hex string
        Dim mytemp As Byte
        Dim mystemp As String

        mytemp = DecNum
        mystemp = "00" & Convert.ToString(mytemp, 16).ToLower
        mystemp = Microsoft.VisualBasic.Right(mystemp, 2)
        Return mystemp
    End Function
    Public Sub OptLoad()
        Try
            FileOpen(1, System.IO.Path.Combine(Application.StartupPath, "myoption.txt"), OpenMode.Input)
            'Public AD9850RefClk As Integer = 124998925
            'Public AD9851RefClk As Integer = 179987000
            '' Default divisor of 10 assumes ad8307 at 25mv per dB and 2.56 volt = 1024 => /10 gives approx dB
            'Public AD8307Div As Double = 9.99023
            'Public AD8307Off As Double = -88.0

            Input(1, AD9850RefClk)
            Input(1, AD9851RefClk)
            For j = 0 To 2
                Input(1, AD8307Div(j))
                Input(1, AD8307Off(j))
            Next


            FileClose(1)
        Catch ex As Exception
            'No or bad file load defaults
            FileClose(1)
            AD9850RefClk = 124998925
            AD9851RefClk = 179987000
            '' Default divisor of 10 assumes ad8307 at 25mv per dB and 2.56 volt = 1024 => /10 gives approx dB
            AD8307Div(0) = 9.9612
            AD8307Off(0) = -89.8225
            AD8307Div(1) = 9.829
            AD8307Off(1) = -89.3234
            AD8307Div(2) = 9.7734
            AD8307Off(2) = -88.0083
        End Try


    End Sub
    Public Sub OptSave()
        Try
            FileOpen(1, System.IO.Path.Combine(Application.StartupPath, "myoption.txt"), OpenMode.Output)

            Printcr(1, AD9850RefClk)
            Printcr(1, AD9851RefClk)
            For j = 0 To 2
                Printcr(1, AD8307Div(j))
                Printcr(1, AD8307Off(j))
            Next


            FileClose(1)
        Catch ex As Exception
            MsgBox(ex.Message)
            FileClose(1)

        End Try


    End Sub
    Public Sub ScreenOptLoad()
        Try
            FileOpen(1, System.IO.Path.Combine(Application.StartupPath, "mychannel.txt"), OpenMode.Input)
            Dim colorval As Integer

            Input(1, colorval)
            ScreenColor = Color.FromArgb(colorval)
            Input(1, colorval)
            GraticuleColor = Color.FromArgb(colorval)
            Input(1, colorval)
            MarkerColor = Color.FromArgb(colorval)
            Input(1, CurrentCurve(0).MeActive)
            Input(1, CurrentCurve(1).MeActive)
            Input(1, CurrentCurve(2).MeActive)
            Input(1, colorval)
            CurrentCurve(0).MeColor = Color.FromArgb(colorval)
            Input(1, colorval)
            CurrentCurve(1).MeColor = Color.FromArgb(colorval)
            Input(1, colorval)
            CurrentCurve(2).MeColor = Color.FromArgb(colorval)
            Input(1, CurrentCurve(0).MeName)
            Input(1, CurrentCurve(1).MeName)
            Input(1, CurrentCurve(2).MeName)
            Input(1, CurrentCurve(0).MeOFF)
            Input(1, CurrentCurve(1).MeOFF)
            Input(1, CurrentCurve(2).MeOFF)
            Input(1, CurrentCurve(0).MeLevel)
            Input(1, CurrentCurve(1).MeLevel)
            Input(1, CurrentCurve(2).MeLevel)
            Input(1, VScale)
            Input(1, VOffset)
            Input(1, ShowVSWR)
            Input(1, LevelRef)
            FileClose(1)
        Catch ex As Exception
            'No or bad file load defaults
            FileClose(1)
            ScreenColor = Color.FromArgb(0, 64, 0)
            GraticuleColor = Color.Black
            MarkerColor = Color.AntiqueWhite
            CurrentCurve(0).MeActive = True
            CurrentCurve(1).MeActive = False
            CurrentCurve(2).MeActive = False
            CurrentCurve(0).MeColor = Color.LightGreen
            CurrentCurve(1).MeColor = Color.LightSteelBlue
            CurrentCurve(2).MeColor = Color.LightSalmon
            CurrentCurve(0).MeName = "Channel 0"
            CurrentCurve(1).MeName = "Channel 1"
            CurrentCurve(2).MeName = "Channel 2"
            CurrentCurve(0).MeOFF = 0
            CurrentCurve(1).MeOFF = 0
            CurrentCurve(2).MeOFF = 0
            CurrentCurve(0).MeLevel = False
            CurrentCurve(1).MeLevel = False
            CurrentCurve(2).MeLevel = False
            VScale = 10
            VOffset = -30
            ShowVSWR = False
            LevelRef = 1
        End Try


    End Sub
    Public Sub ScreenOptSave()
        Try
            FileOpen(1, System.IO.Path.Combine(Application.StartupPath, "mychannel.txt"), OpenMode.Output)

            Printcr(1, ScreenColor.ToArgb.ToString)
            Printcr(1, GraticuleColor.ToArgb.ToString)
            Printcr(1, MarkerColor.ToArgb.ToString)
            Printcr(1, CurrentCurve(0).MeActive)
            Printcr(1, CurrentCurve(1).MeActive)
            Printcr(1, CurrentCurve(2).MeActive)
            Printcr(1, CurrentCurve(0).MeColor.ToArgb.ToString)
            Printcr(1, CurrentCurve(1).MeColor.ToArgb.ToString)
            Printcr(1, CurrentCurve(2).MeColor.ToArgb.ToString)
            Printcr(1, CurrentCurve(0).MeName)
            Printcr(1, CurrentCurve(1).MeName)
            Printcr(1, CurrentCurve(2).MeName)
            Printcr(1, CurrentCurve(0).MeOFF)
            Printcr(1, CurrentCurve(1).MeOFF)
            Printcr(1, CurrentCurve(2).MeOFF)
            Printcr(1, CurrentCurve(0).MeLevel)
            Printcr(1, CurrentCurve(1).MeLevel)
            Printcr(1, CurrentCurve(2).MeLevel)
            Printcr(1, VScale)
            Printcr(1, VOffset)
            Printcr(1, ShowVSWR)
            Printcr(1, LevelRef)
            FileClose(1)
        Catch ex As Exception
            MsgBox(ex.Message)
            FileClose(1)

        End Try


    End Sub
    Public Sub Printcr(ByVal fn As Integer, ByVal strin As String)
        Dim pstr As String
        pstr = strin & vbCrLf
        Print(fn, pstr)
    End Sub
End Module
