﻿Public Class sweeper

    ' Basic Frequency Sweeping Application for VK3DIP Generic Interface using an AD9850 or AD9851
    ' Paul McMahon VK3DIP 2014
    ' Initial Version 1.0  June 2014
    ' Added Markers and Calibration Forms (V1.1) November 2014
    ' Added AD9851/9850 Option January 2015 V2.0
    ' Multi Trace V4 February 2015
    ' V5 did multi trace simultaniously
    ' V5.1 Different AD8307 constants for each channel
    ' V6  March 2015 removed extra differeence channels and added in optional leveling of any channel
    ' V6.1 24/03/2015 Added resize to main and markers forms
    Public minsize As Size


#Region "Form Start and Stop"


    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ' set version
        Me.Text = "VK3DIP Generic Sweeper V" & MyVersion
        minsize = Me.Size

        ddBaud.Items.Add("4800")
        ddBaud.Items.Add("9600")
        ddBaud.Items.Add("19200")
        ddBaud.Items.Add("38400")
        ddBaud.Items.Add("57600")
        ddBaud.Items.Add("115200")

        ddBaud.SelectedIndex = 4 ' default to 57600

        ' preload the available comm ports and set to first found
        ' or error msg if none found

        PortSelect.Items.Clear()
        For Each portName As String In My.Computer.Ports.SerialPortNames
            PortSelect.Items.Add(portName)
        Next

        Try
            PortSelect.SelectedIndex = 0
            btnCom.Text = "Start"
        Catch es As Exception
            btnCom.Text = "OOPS"
            MsgBox("No free ports found")
        End Try
        AD9851CB.Checked = AD9851Flg

        OptLoad() ' Load the calibration constants

        If AD9851Flg Then
            MaxNUD.Value = 60000000
        Else
            MaxNUD.Value = 30000000
        End If

        ScreenWidth = CROScreen.Width
        ScreenHeight = CROScreen.Height

        For j = 0 To 2
            CurrentCurve(j).NumPoints = 0
            ReDim CurrentCurve(j).Markers(maxmarkers - 1)
            CurrentCurve(j).NumMarkers = 0
        Next
        ScreenOptLoad()
       
            P0NameLab.ForeColor = CurrentCurve(0).MeColor
            P1NameLab.ForeColor = CurrentCurve(1).MeColor
        P2NameLab.ForeColor = CurrentCurve(2).MeColor
        C0Group.ForeColor = CurrentCurve(0).MeColor
        C1Group.ForeColor = CurrentCurve(1).MeColor
        C2Group.ForeColor = CurrentCurve(2).MeColor
        'C0Group.BackColor = ScreenColor
        'C1Group.BackColor = ScreenColor
        'C2Group.BackColor = ScreenColor
        P0NameLab.BackColor = ScreenColor
        P1NameLab.BackColor = ScreenColor
        P2NameLab.BackColor = ScreenColor
            P0NameLab.Text = CurrentCurve(0).MeName
            P1NameLab.Text = CurrentCurve(1).MeName
            P2NameLab.Text = CurrentCurve(2).MeName
            P0NameLab.Visible = CurrentCurve(0).MeActive
            C0Group.Enabled = CurrentCurve(0).MeActive
            C1Group.Text = "Channel 1"
            P1NameLab.Visible = CurrentCurve(1).MeActive
            C1Group.Enabled = CurrentCurve(1).MeActive
            P2NameLab.Visible = CurrentCurve(2).MeActive
            C2Group.Enabled = CurrentCurve(2).MeActive


        CROScreen.BackColor = ScreenColor
        A2DAvgNUD.Value = A2DAvg
        If CurMode = 0 Then SingleRB.Checked = True
        normalizeoffAll()
        UpdateScales()
    End Sub

    Private Sub ExitBut_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ExitBut.Click
        Me.Close()
    End Sub

#End Region

#Region "Paint Screen"

    Private Sub PictureBox1_Paint(ByVal sender As Object, ByVal e As System.Windows.Forms.PaintEventArgs) Handles CROScreen.Paint
        Mydogrid(e)
    End Sub
    Private Sub Mydogrid(ByVal e As System.Windows.Forms.PaintEventArgs)
        Dim MajGP As Pen = New Pen(GraticuleColor, 1)
        Dim CurveP(2) As Pen
        Dim mygridcnt As Single
        Dim mystepw As Single
        Dim mysteph As Single
        Dim doit As Boolean = False
        ' Nominal croscreen size is 400 wide (0 to 399) and 320 High ( 319 to 0) 
        ScreenWidth = CROScreen.Width
        ScreenHeight = CROScreen.Height
        For j = 0 To 2
            CurveP(j) = New Pen(CurrentCurve(j).MeColor, 2)
        Next
        mystepw = ScreenWidth / 10
        mysteph = ScreenHeight / 8

        ' Do Maj vert. grid lines
        For mygridcnt = 0 To Me.Width Step mystepw
            e.Graphics.DrawLine(MajGP, mygridcnt, 0, mygridcnt, CSng(CROScreen.Height))
        Next
        ' Do Maj horiz. grid lines
        For mygridcnt = 0 To Me.Height Step mysteph
            e.Graphics.DrawLine(MajGP, 0, mygridcnt, CSng(CROScreen.Width), mygridcnt)
        Next
        For j = 0 To 2
            With CurrentCurve(j)
                If .NumPoints > 1 And .MeActive Then
                    e.Graphics.DrawLines(CurveP(j), .Ps)
                    If .NumMarkers > 0 And MarkersOn(j) Then
                        ' Create font and brush.
                        Dim drawFont As New Font("Arial", 12)
                        Dim drawBrush As New SolidBrush(.MeColor)
                        ' Set format of string.
                        Dim drawFormat As New StringFormat
                        drawFormat.FormatFlags = StringFormatFlags.DirectionRightToLeft
                        Dim Crosspen As New Pen(MarkerColor, 1)
                        Dim CrossSize As Single = 5.0

                        For i = 0 To .NumMarkers - 1
                            ' if we have a point to put the marker on then show it else ignore it
                            If .Markers(i) < .NumPoints Then
                                ' Create point for upper-left corner of drawing.
                                Dim drawPoint As New PointF(CSng(.Ps(.Markers(i)).X), CSng(.Ps(.Markers(i)).Y))

                                ' Draw string to screen.
                                e.Graphics.DrawString((i + 1).ToString("0"), drawFont, drawBrush, drawPoint, drawFormat)
                                ' Do crosses

                                ' Create points that define lines.
                                Dim point1 As New PointF(CSng(.Ps(.Markers(i)).X) - CrossSize, CSng(.Ps(.Markers(i)).Y) + CrossSize)
                                Dim point2 As New PointF(CSng(.Ps(.Markers(i)).X) + CrossSize, CSng(.Ps(.Markers(i)).Y) - CrossSize)
                                Dim point3 As New PointF(CSng(.Ps(.Markers(i)).X) - CrossSize, CSng(.Ps(.Markers(i)).Y) - CrossSize)
                                Dim point4 As New PointF(CSng(.Ps(.Markers(i)).X) + CrossSize, CSng(.Ps(.Markers(i)).Y) + CrossSize)

                                ' Draw cross to screen.
                                e.Graphics.DrawLine(Crosspen, point1, point2)
                                e.Graphics.DrawLine(Crosspen, point3, point4)
                            End If

                        Next
                        Crosspen.Dispose()
                        drawFont.Dispose()
                        drawBrush.Dispose()
                    End If
                End If
            End With
        Next
        ' cleanup just in case
        CurveP(0).Dispose()
        CurveP(1).Dispose()
        CurveP(2).Dispose()
        MajGP.Dispose()
    End Sub

   
#End Region

#Region "Serial Funtions"


    Public Function SendReceive(ByVal strout As String) As String
        If SerialPort1.IsOpen Then
            Dim myresult As Integer = 0

            Try
                SerialPort1.WriteLine(strout)
                SendReceive = SerialPort1.ReadLine
            Catch ex As Exception
                Call stopit()
                checkport()
                MsgBox("Port Timed Out, Check Interface")
                SendReceive = "0"
            End Try
        Else
            SendReceive = "0"
        End If
    End Function

  
 

    Public Function ReadMultiVatFa(ByVal Freq As Double, ByVal Ravg As Integer) As TriDouble
        'Sets the DDS frequency to Freq, then reads (for all active channels) the sum of Ravg voltage measurements.
        'Converts sum to equiv dB reading using ADdiv and ADoff then returns
        Dim tempresult As TriDouble
        Dim myresult As Integer = 0
        Dim myavg As Double
        Dim TempS As String = ""
        Dim SendCmd As String

        ' Set Frequency of AD9850
        If SportsMode Then
            TempS = SendReceive(Phase2String(Freq2Phase(Freq)).Replace(",", "").Replace(" ", ""))
        Else
            TempS = SendReceive(Phase2String(Freq2Phase(Freq)))
        End If

        ' Read Voltage of AD8307
        'AD8307Div(2) 
        'AD8307Off(2)
        For j = 0 To 2
            If CurrentCurve(j).MeActive Or LevelRef = j Then
                Select Case j
                    Case 0
                        If SportsMode Then
                            SendCmd = "A00" & BInt2Hex(Ravg) ' Ask for a average voltage (really the sum of Ravg readings)
                        Else
                            SendCmd = "A 00," & BInt2Hex(Ravg) ' Ask for a average voltage (really the sum of Ravg readings)
                        End If
                        TempS = SendReceive(SendCmd)
                        myresult = String2Volt(TempS) ' convert the received string with 4 digit hex sum to a integer
                        myavg = CDbl(myresult) / CDbl(Ravg) ' actual average reading
                        tempresult.D0 = myavg / AD8307Div(0) + AD8307Off(0) ' scale/convert to db 
                        If tempresult.D0 > 11 Then
                            MsgBox("Warning - Level Exceeding Linear Range Channel 0 = " & tempresult.D0.ToString("#0.#dB"))
                        End If
                    Case 1
                        If SportsMode Then
                            SendCmd = "A01" & BInt2Hex(Ravg) ' Ask for a average voltage (really the sum of Ravg readings)
                        Else
                            SendCmd = "A 01," & BInt2Hex(Ravg) ' Ask for a average voltage (really the sum of Ravg readings)
                        End If
                        TempS = SendReceive(SendCmd)
                        myresult = String2Volt(TempS) ' convert the received string with 4 digit hex sum to a integer
                        myavg = CDbl(myresult) / CDbl(Ravg) ' actual average reading
                        tempresult.D1 = myavg / AD8307Div(1) + AD8307Off(1) ' scale/convert to db 
                        If tempresult.D1 > 11 Then
                            MsgBox("Warning - Level Exceeding Linear Range Channel 1= " & tempresult.D1.ToString("#0.#dB"))
                        End If
                    Case 2
                        If SportsMode Then
                            SendCmd = "A02" & BInt2Hex(Ravg) ' Ask for a average voltage (really the sum of Ravg readings)
                        Else
                            SendCmd = "A 02," & BInt2Hex(Ravg) ' Ask for a average voltage (really the sum of Ravg readings)
                        End If
                        TempS = SendReceive(SendCmd)
                        myresult = String2Volt(TempS) ' convert the received string with 4 digit hex sum to a integer
                        myavg = CDbl(myresult) / CDbl(Ravg) ' actual average reading
                        tempresult.D2 = myavg / AD8307Div(2) + AD8307Off(2) ' scale/convert to db 
                        If tempresult.D2 > 11 Then
                            MsgBox("Warning - Level Exceeding Linear Range Channel 2= " & tempresult.D2.ToString("#0.#dB"))
                        End If
                    Case Else

                End Select
            End If
            ReadMultiVatFa = tempresult
        Next

    End Function
#End Region

#Region "Commport Select"
    Private Sub portselect_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs)
        If SerialPort1.IsOpen Then
            SerialPort1.Close()
            btnCom.Text = "Start"
            'Timer1.Enabled = False
            ' MyWaitforRX = True
        End If
    End Sub
    Private Sub ddBaud_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ddBaud.SelectedIndexChanged
        If SerialPort1.IsOpen Then
            Call stopit()
        End If
    End Sub
    Private Sub btnRefresh_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnRefresh.Click

        If SerialPort1.IsOpen Then
            Call stopit()
        End If

        PortSelect.Items.Clear()
        For Each portName As String In My.Computer.Ports.SerialPortNames
            PortSelect.Items.Add(portName)
        Next
        Try
            PortSelect.SelectedIndex = 0
            btnCom.Text = "Start"
        Catch es As Exception
            btnCom.Text = "OOPS"
            MessageBox.Show("No free ports found")
        End Try

        checkport()
    End Sub
    ' to Start & Stop
    Private Sub btnCom_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnCom.Click
        If btnCom.Text = "OOPS" Then
            MsgBox("No free ports found")
        Else
            If btnCom.Text = "Start" Then
                Call startit()
            Else ' must be stop
                Call stopit()
            End If
        End If
        checkport()
    End Sub
    Private Sub stopit()
        If SerialPort1.IsOpen Then
            Try
                SerialPort1.Close()
                btnCom.Text = "Start"
              
            Catch ex As Exception
                MsgBox(ex.ToString)
            End Try
        End If
    End Sub
    Private Sub startit()
        If SerialPort1.IsOpen Then
            SerialPort1.Close()
        End If
        'Timer1.Enabled = False
        'MyWaitforRX = True
        Try
            With SerialPort1
                .PortName = PortSelect.Text
                .BaudRate = ddBaud.Text
                .Handshake = IO.Ports.Handshake.None
                .Parity = IO.Ports.Parity.None
                .DataBits = 8
                .StopBits = IO.Ports.StopBits.One
                .ReadTimeout = 5000 ' 5 seconds just in case
            End With
            SerialPort1.Open()

            btnCom.Text = "Stop"
            Dim TempS As String = ""

        Catch ex As Exception
            MsgBox(ex.ToString)
        End Try
    End Sub
#End Region

#Region "Serial Pin Change Events"


    'sub to handle the serial port pin changed event
    Private Sub SerialPort1_PinChanged(ByVal sender As System.Object, ByVal e As System.IO.Ports.SerialPinChangedEventArgs) Handles SerialPort1.PinChanged
        '    'invoke delegate to cross from serial tread to form thread
        Me.Invoke(New myDelegate(AddressOf checkport), New Object() {})

    End Sub
 
    Public Delegate Sub myDelegate()
    Public Sub checkport()
        If SerialPort1.IsOpen Then
            LabLed.BackColor = Color.GreenYellow
            CommsUp = True
            ' RS232 Signals
            If SerialPort1.CDHolding Then
                labCD.BackColor = Color.GreenYellow
            Else
                labCD.BackColor = Color.Red
            End If
            If SerialPort1.CtsHolding Then
                LabCTS.BackColor = Color.GreenYellow
            Else
                LabCTS.BackColor = Color.Red
            End If
            If SerialPort1.DsrHolding Then
                LabDSR.BackColor = Color.GreenYellow
            Else
                LabDSR.BackColor = Color.Red
            End If
        Else
            LabLed.BackColor = Color.Red
            CommsUp = False
        End If
    End Sub
#End Region

#Region "Menu Items"

    Private Sub AD9850ToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles AD9850ToolStripMenuItem.Click
        Dim res As Integer
        stopsweep()
        normalizeoffAll()
        If CommsUp Then
            If AD9851Flg Then
                AD9850Cal.FClkNUD.Value = AD9851RefClk
            Else
                AD9850Cal.FClkNUD.Value = AD9850RefClk
            End If
            res = AD9850Cal.ShowDialog()
            If res = System.Windows.Forms.DialogResult.OK Then
                OptSave()
            End If
        Else
            MsgBox(" Start Comms First")
        End If
        CurMode = 0
    End Sub

    Private Sub AD8307ToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles AD8307ToolStripMenuItem.Click
        Dim res As DialogResult
        stopsweep()
        normalizeoffAll()
        If CommsUp Then
            res = AD8307Cal.ShowDialog()
            If res = System.Windows.Forms.DialogResult.OK Then
                OptSave()
                A2DAvgNUD.Value = A2DAvg
            End If
        Else
            MsgBox(" Start Comms First")
        End If
        CurMode = 0
    End Sub

    Private Sub ChannelsToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ChannelsToolStripMenuItem.Click
        Dim res As DialogResult

        res = Channels.ShowDialog()
        If res = System.Windows.Forms.DialogResult.OK Then
            P0NameLab.ForeColor = CurrentCurve(0).MeColor
                P1NameLab.ForeColor = CurrentCurve(1).MeColor
            P2NameLab.ForeColor = CurrentCurve(2).MeColor
            C0Group.ForeColor = CurrentCurve(0).MeColor
            C1Group.ForeColor = CurrentCurve(1).MeColor
            C2Group.ForeColor = CurrentCurve(2).MeColor
            'C0Group.BackColor = ScreenColor
            'C1Group.BackColor = ScreenColor
            'C2Group.BackColor = ScreenColor
            P0NameLab.BackColor = ScreenColor
            P1NameLab.BackColor = ScreenColor
            P2NameLab.BackColor = ScreenColor
                P0NameLab.Text = CurrentCurve(0).MeName
                P1NameLab.Text = CurrentCurve(1).MeName
                P2NameLab.Text = CurrentCurve(2).MeName
                P0NameLab.Visible = CurrentCurve(0).MeActive
                C0Group.Enabled = CurrentCurve(0).MeActive
                C1Group.Text = "Channel 1"
                P1NameLab.Visible = CurrentCurve(1).MeActive
                C1Group.Enabled = CurrentCurve(1).MeActive
                P2NameLab.Visible = CurrentCurve(2).MeActive
                C2Group.Enabled = CurrentCurve(2).MeActive


            CROScreen.BackColor = ScreenColor
            UpdateScales()
            ScreenOptSave()
            CROScreen.Refresh()
        End If
    End Sub

#End Region

#Region "Sweep Stuff"

    Public Sub stopsweep()
        StrtBut.Text = "Start"
        CurMode = 0
        Application.DoEvents()
    End Sub

    Private Sub StrtBut_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles StrtBut.Click
        If CommsUp Then
            If StrtBut.Text = "Start" Then
                StrtBut.Text = "Stop"
                CurMode = 1
            Else
                stopsweep()
            End If
            Application.DoEvents()
        Else
            MsgBox("Start Comms First")
        End If
    End Sub

    Private Sub ContinuousRB_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ContinuousRB.CheckedChanged
        ChangeSweep()
    End Sub

    Private Sub SingleRB_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SingleRB.CheckedChanged
        ChangeSweep()
    End Sub

    Private Sub ChangeSweep()
        stopsweep()
        If ContinuousRB.Checked Then
            StrtBut.Enabled = True
            GoBut.Enabled = False
        Else
            StrtBut.Enabled = False
            GoBut.Enabled = True
        End If
    End Sub

    Public Sub DoSweep()
        Dim TempMax As Double
        Dim TempMin As Double
        Dim TempSteps As Integer
        Dim stepsize As Double
        Dim Frequency As Double
        Dim myindex As Integer
        Dim Foffset As Double
        Dim LevelOff As Double
        Dim RefreshCnt As Integer
        Dim Refreshtrig As Integer
        Dim tempresult As TriDouble
        Dim millisec As Long
        Dim TransperSec As Double
        Dim mywatch As New System.Diagnostics.Stopwatch
        If CommsUp Then
            TempMax = MaxNUD.Value
            TempMin = MinNUD.Value
            TempSteps = StepsNUD.Value
            Refreshtrig = CInt(TempSteps / RefNUD.Value)
            stepsize = (TempMax - TempMin) / TempSteps
            Foffset = FreqOffNUD.Value
            LevelOff = Channels.LevOffNUD.Value
            For j = 0 To 2
                If CurrentCurve(j).MeActive Or LevelRef = j Then
                    ReDim CurrentCurve(j).Values(TempSteps + 1) 'make enough room for the result of a sweep
                End If
            Next
            myindex = 0
            mywatch.Start()
            SLedLab.BackColor = Color.GreenYellow
            SLedLab.Refresh()
            Application.DoEvents()
            RefreshCnt = 0
            For Frequency = TempMin To TempMax + stepsize / 2 Step stepsize
                tempresult = ReadMultiVatFa(Frequency, A2DAvg)
                For j = 0 To 2
                    If CurrentCurve(j).MeActive Or LevelRef = j Then
                        CurrentCurve(j).Values(myindex).RF = Frequency
                        Select Case j
                            Case 0
                                CurrentCurve(j).Values(myindex).RL = tempresult.D0
                            Case 1
                                CurrentCurve(j).Values(myindex).RL = tempresult.D1
                            Case 2
                                CurrentCurve(j).Values(myindex).RL = tempresult.D2
                        End Select
                        CurrentCurve(j).Values(myindex).DF = Frequency + Foffset
                    End If
                Next
                For j = 0 To 2
                    If CurrentCurve(j).MeActive Then
                        CurrentCurve(j).Values(myindex).DL = LevelAndNormalize(myindex, j)
                    End If
                Next
                myindex = myindex + 1
                RefreshCnt = RefreshCnt + 1
                If RefreshCnt > Refreshtrig Then
                    RefreshCnt = 0
                    ShowWhatUGotAll(myindex)
                End If
            Next
            If RefreshCnt <> 0 Then
                RefreshCnt = 0
                ShowWhatUGotAll(myindex)
            End If
            mywatch.Stop()
            SLedLab.BackColor = Color.Red
            millisec = mywatch.ElapsedMilliseconds
            TransperSec = 1000 * TempSteps / millisec
            TRateL.Text = TransperSec.ToString("###0.0 TPS")
            SLedLab.Refresh()
            Application.DoEvents()
            updatemarkers()
        Else
            MsgBox("Start Comms First")
            stopsweep()
        End If
    End Sub

    Private Sub GoBut_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles GoBut.Click
        DoSweep()
    End Sub
#End Region

#Region "Screen Stuff"
    Private Sub UpdateScales()
        FDL = MinNUD.Value + FreqOffNUD.Value
        FDH = MaxNUD.Value + FreqOffNUD.Value
        StartFreqLab.Text = FDL.ToString("###,###,##0")
        HorizScaleLab.Text = ((MaxNUD.Value - MinNUD.Value) / 10).ToString("###,###,##0") & " Hz per Division"
        StopFreqLab.Text = FDH.ToString("###,###,##0")
        setvert()
    End Sub

    Private Sub ShowWhatUGot(ByVal npoints As Integer, ByVal curveno As Integer)

        If CurrentCurve(curveno).MeActive Then
            CurrentCurve(curveno).NumPoints = npoints
            MakePoints(CurrentCurve(curveno))
        End If

        CROScreen.Refresh()
    End Sub
    Private Sub ShowWhatUGotAll(ByVal npoints As Integer)
        For j = 0 To 2
            If CurrentCurve(j).MeActive Then
                CurrentCurve(j).NumPoints = npoints
                MakePoints(CurrentCurve(j))
            End If
        Next
        CROScreen.Refresh()
    End Sub

    Private Sub RecalcAll()
        For j = 0 To 2
            If CurrentCurve(j).MeActive Then
                MakePoints(CurrentCurve(j))
            End If
        Next
        CROScreen.Refresh()
    End Sub

    Private Sub CROScreen_MouseLeave(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CROScreen.MouseLeave
        updatemarkers()
        CrosshairLab.Text = ""
        Cursor = Cursors.Default
    End Sub
    Private Sub CROScreen_MouseMove(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles CROScreen.MouseMove
        Dim myDL As Double
        Dim myDF As Double
        Dim myVSWR As Double
        Dim myGAMMA As Double

        myDL = (e.Y * (LDL - LDH) + ScreenHeight * LDH) / ScreenHeight
        myDF = -(e.X * (FDL - FDH) - ScreenWidth * FDL) / ScreenWidth

        If ShowVSWR Then
            myGAMMA = Math.Abs(10 ^ (-myDL / 20))
            myVSWR = (myGAMMA + 1) / (myGAMMA - 1)
            CrosshairLab.Text = myDF.ToString("###,###,##0Hz") & "  " & myDL.ToString("##0.00dB") & " (" & myVSWR.ToString("#0.000") & ":1)"
        Else
            CrosshairLab.Text = myDF.ToString("###,###,##0Hz") & "  " & myDL.ToString("##0.00dB")
        End If


    End Sub
    Private Sub CROScreen_MouseEnter(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles CROScreen.MouseEnter
        Cursor = Cursors.Cross
    End Sub

    Private Function overpoint(ByVal curpoint As Point, ByVal curveno As Integer) As Integer
        ' find nearest point in current curve return index else if too far away return 0
        Dim maxrange2 As Integer = 100 ' squared maximum range
        Dim D2 As Integer
        Dim MinD2 As Integer = 110
        Dim MinIndex As Integer = 0
        overpoint = 0
        For i = 1 To CurrentCurve(curveno).NumPoints - 1
            D2 = (curpoint.X - CurrentCurve(curveno).Ps(i).X) ^ 2 + (curpoint.Y - CurrentCurve(curveno).Ps(i).Y) ^ 2
            If D2 < MinD2 Then
                MinD2 = D2
                MinIndex = i
            End If
        Next
        If MinD2 < maxrange2 Then
            overpoint = MinIndex
        End If
    End Function


    Private Sub CROScreen_MouseClick(ByVal sender As System.Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles CROScreen.MouseClick

        Dim curindex As Integer
        Dim myresult As Integer

        Dim mp As Point = CROScreen.PointToClient(MousePosition)
        For j = 0 To 2
            curindex = overpoint(mp, j)
            If curindex > 0 Then
                myresult = SetResetmarker(curindex, j)

            End If
        Next
        updatemarkers()
        CROScreen.Refresh()

    End Sub
   


    Private Sub setvert()
        Dim Vertstep As Integer = ScreenHeight / 8
        VertScaleLab.Text = VScale.ToString("##0")
        LDH = VScale * 4 + VOffset
        LDL = VScale * -4 + VOffset

        VLab0.Text = Vscaletext(0)
        VLab0.Left = CROScreen.Location.X - VLab0.Size.Width
        VLab0.Top = CROScreen.Location.Y - VLab0.Size.Height / 2 + 8 * Vertstep

        VLab1.Text = Vscaletext(1)
        VLab1.Left = CROScreen.Location.X - VLab1.Size.Width
        VLab1.Top = CROScreen.Location.Y - VLab1.Size.Height / 2 + 7 * Vertstep

        VLab2.Text = Vscaletext(2)
        VLab2.Left = CROScreen.Location.X - VLab2.Size.Width
        VLab2.Top = CROScreen.Location.Y - VLab2.Size.Height / 2 + 6 * Vertstep

        VLab3.Text = Vscaletext(3)
        VLab3.Left = CROScreen.Location.X - VLab3.Size.Width
        VLab3.Top = CROScreen.Location.Y - VLab3.Size.Height / 2 + 5 * Vertstep

        VLab4.Text = Vscaletext(4)
        VLab4.Left = CROScreen.Location.X - VLab4.Size.Width
        VLab4.Top = CROScreen.Location.Y - VLab4.Size.Height / 2 + 4 * Vertstep

        VLab5.Text = Vscaletext(5)
        VLab5.Left = CROScreen.Location.X - VLab5.Size.Width
        VLab5.Top = CROScreen.Location.Y - VLab5.Size.Height / 2 + 3 * Vertstep

        VLab6.Text = Vscaletext(6)
        VLab6.Left = CROScreen.Location.X - VLab6.Size.Width
        VLab6.Top = CROScreen.Location.Y - VLab6.Size.Height / 2 + 2 * Vertstep

        VLab7.Text = Vscaletext(7)
        VLab7.Left = CROScreen.Location.X - VLab7.Size.Width
        VLab7.Top = CROScreen.Location.Y - VLab7.Size.Height / 2 + Vertstep

        VLab8.Text = Vscaletext(8)
        VLab8.Left = CROScreen.Location.X - VLab8.Size.Width
        VLab8.Top = CROScreen.Location.Y - VLab8.Size.Height / 2

    End Sub

    Private Function Vscaletext(ByVal pnum As Integer) As String
        Vscaletext = VScale * (pnum - 4) + VOffset.ToString("##0")
    End Function

#End Region

#Region "Frequency Change"

    Private Sub MinNUD_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MinNUD.ValueChanged
        clearallmarkers()
        UpdateScales()
        normalizeoffAll()
    End Sub

    Private Sub MaxNUD_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MaxNUD.ValueChanged
        clearallmarkers()
        UpdateScales()
        normalizeoffAll()
    End Sub


    Private Sub FreqOffNUD_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles FreqOffNUD.ValueChanged
        UpdateScales()
    End Sub

    Private Sub StepsNUD_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles StepsNUD.ValueChanged
        clearallmarkers()
    End Sub


    Private Sub DefFreqB_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles DefFreqB.Click
        If AD9851Flg Then
            MaxNUD.Value = 60000000
        Else
            MaxNUD.Value = 30000000
        End If
        MinNUD.Value = 1000000
        clearallmarkers()
        UpdateScales()
        normalizeoffAll()
    End Sub

#End Region

#Region "Main Timer"
    Private Sub Mytimer_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyTimer.Tick
        'Main Background timer tick
        ' Depending on current mode do stuff.
        ' First stop timer to prevent retriggers
        MyTimer.Enabled = False
        If SportsMode Then
            MyTimer.Interval = 250
        Else
            MyTimer.Interval = 1000
        End If
        If InEdit Then
            stopsweep()
        Else
            If CommsUp Then
                Select Case CurMode
                    Case 0 ' Single Sweep mode do nothing
                    Case 1 ' Continuous sweep mode trigger a new sweep
                        DoSweep()
                    Case 2 ' Doing AD9850 Cal
                        Dim Deltaphase As UInt64
                        Dim myresult As String = ""
                        Dim tempD As Double
                        Deltaphase = (AD9850Cal.FReqNUD.Value * Math.Pow(2, 32)) / AD9850Cal.FClkNUD.Value
                        myresult = Phase2String(Deltaphase)
                        AD9850Cal.ResultsL.Text = "Command = " & myresult
                        tempD = Deltaphase * AD9850Cal.FClkNUD.Value / Math.Pow(2, 32)
                        AD9850Cal.RealFreqLab.Text = "Exact Freq.=" & tempD.ToString("#######0.000Hz")
                        myresult = SendReceive(myresult)
                        AD9850Cal.Invalidate()
                    Case 3 ' Doing AD8307 Cal freerun
                        Dim TempS As String = ""
                        Dim SendCmd As String
                        Dim mydb As Double
                        Dim mymW As Double
                        Dim myV As Double
                        Select Case AD8307Cal.PortNud.Value
                            Case 0
                                SendCmd = "A00," & BInt2Hex(AD8307Cal.AvgNUD.Value) ' Ask for a average voltage (really the sum of Ravg readings)
                            Case 1
                                SendCmd = "A01," & BInt2Hex(AD8307Cal.AvgNUD.Value) ' Ask for a average voltage (really the sum of Ravg readings)
                            Case 2
                                SendCmd = "A02," & BInt2Hex(AD8307Cal.AvgNUD.Value) ' Ask for a average voltage (really the sum of Ravg readings)
                            Case Else
                                SendCmd = "A00," & BInt2Hex(AD8307Cal.AvgNUD.Value) ' Ask for a average voltage (really the sum of Ravg readings)
                        End Select
                        TempS = SendReceive(SendCmd)
                        mydb = String2Volt(TempS)
                        mydb = CDbl(mydb) / (AD8307Cal.AD8307ScaleNUD.Value * AD8307Cal.AvgNUD.Value) + AD8307Cal.AD8307ConstNUD.Value
                        AD8307Cal.dBmLab.Text = mydb.ToString("##0.00 dBm")
                        mymW = 10 ^ (mydb / 10)
                        TempS = mymW.ToString("###0.000 mW")
                        myV = Math.Sqrt(50 * mymW / 1000)
                        AD8307Cal.mWVLab.Text = TempS & " / " & (myV * 1000).ToString("###0.000 mV") & " @ 50 Ohms"
                        AD8307Cal.Invalidate()
                    Case Else ' I don't know so do nothing

                End Select
            End If
        End If
        ' Re enable timer
        MyTimer.Enabled = True

    End Sub
#End Region

#Region "Curve Stuff"
    Private Sub Save0But_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Save0But.Click
        If CurrentCurve(0).NumPoints > 1 And CurrentCurve(0).MeActive Then
            SavedCurve(0) = CurrentCurve(0)
            Save0But.BackColor = Color.GreenYellow
        End If
    End Sub

    Private Sub Save1But_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Save1But.Click
        If C1Group.Text = "Channel 1" Then
            If CurrentCurve(1).NumPoints > 1 And CurrentCurve(1).MeActive Then
                SavedCurve(1) = CurrentCurve(1)
                Save1But.BackColor = Color.GreenYellow
            End If
        Else
            If CurrentCurve(3).NumPoints > 1 And CurrentCurve(3).MeActive Then
                SavedCurve(3) = CurrentCurve(3)
                Save1But.BackColor = Color.GreenYellow
            End If
        End If


    End Sub

    Private Sub Save2But_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Save2But.Click
        If CurrentCurve(2).NumPoints > 1 And CurrentCurve(2).MeActive Then
            SavedCurve(2) = CurrentCurve(2)
            Save2But.BackColor = Color.GreenYellow
        End If
    End Sub

    Private Sub Norm0CB_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Norm0CB.CheckedChanged
        If Norm0CB.Checked Then
            If Save0But.BackColor = Color.AntiqueWhite Then
                Norm0CB.Checked = False ' You need to have saved a curve to normalize against
                MsgBox("Save a base curve first")
            Else
                NormalizeIt(0) = True
            End If
        Else
            NormalizeIt(0) = False
        End If
    End Sub
    Private Sub Norm1CB_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Norm1CB.CheckedChanged
        If Norm1CB.Checked Then
            If Save1But.BackColor = Color.AntiqueWhite Then
                Norm1CB.Checked = False ' You need to have saved a curve to normalize against
                MsgBox("Save a base curve first")
            Else
                NormalizeIt(1) = True
            End If
        Else
            NormalizeIt(1) = False
        End If
    End Sub

    Private Sub Norm2CB_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Norm2CB.CheckedChanged
        If Norm2CB.Checked Then
            If Save2But.BackColor = Color.AntiqueWhite Then
                Norm2CB.Checked = False ' You need to have saved a curve to normalize against
                MsgBox("Save a base curve first")
            Else
                NormalizeIt(2) = True
            End If
        Else
            NormalizeIt(2) = False
        End If
    End Sub
    Public Sub normalizeoff0()
        Save0But.BackColor = Color.AntiqueWhite
        NormalizeIt(0) = False
        Norm0CB.Checked = False
    End Sub
    Public Sub normalizeoff1()
        Save1But.BackColor = Color.AntiqueWhite
        NormalizeIt(1) = False
        Norm1CB.Checked = False
    End Sub
    Public Sub normalizeoff2()
        Save2But.BackColor = Color.AntiqueWhite
        NormalizeIt(2) = False
        Norm2CB.Checked = False
    End Sub
   
    Public Sub normalizeoffAll()
        normalizeoff0()
        normalizeoff1()
        normalizeoff2()
    End Sub
#End Region

#Region "Level stuff"

    Private Sub Level0CB_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Level0CB.CheckedChanged
        If Level0CB.Checked Then
            LevelIt(0) = True

        Else
            LevelIt(0) = False
        End If
    End Sub

    Private Sub Level1CB_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Level1CB.CheckedChanged
        If Level1CB.Checked Then
            LevelIt(1) = True

        Else
            LevelIt(1) = False
        End If
    End Sub

    Private Sub Level2CB_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Level2CB.CheckedChanged
        If Level2CB.Checked Then
            LevelIt(2) = True

        Else
            LevelIt(2) = False
        End If
    End Sub
#End Region

#Region "Marker Stuff"

    Private Sub Marker0CB_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Marker0CB.CheckedChanged
        MarkersOn(0) = Marker0CB.Checked
        If MarkersOn(0) Or MarkersOn(1) Or MarkersOn(2) Then
            Markers.Show()
        Else
            Markers.Hide()
        End If
        clearmarkers(0)
        CROScreen.Refresh()
    End Sub


    Private Sub Marker1CB_CheckedChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles Marker1CB.CheckedChanged
        MarkersOn(1) = Marker1CB.Checked
        If MarkersOn(0) Or MarkersOn(1) Or MarkersOn(2) Then
            Markers.Show()
        Else
            Markers.Hide()
        End If
        clearmarkers(1)
        CROScreen.Refresh()
    End Sub

    Private Sub Marker2CB_CheckedChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles Marker2CB.CheckedChanged
        MarkersOn(2) = Marker2CB.Checked
        If MarkersOn(0) Or MarkersOn(1) Or MarkersOn(2) Then
            Markers.Show()
        Else
            Markers.Hide()
        End If
        clearmarkers(2)
        CROScreen.Refresh()
    End Sub

    Public Sub clearmarkers(ByVal curveno)
        ReDim CurrentCurve(curveno).Markers(maxmarkers - 1)
        CurrentCurve(curveno).NumMarkers = 0
        CROScreen.Refresh()
        updatemarkers()
    End Sub
    Public Sub clearallmarkers()
        For j = 0 To 2
            clearmarkers(j)
        Next
    End Sub
#End Region

#Region "Dont Trigger While editing"
    Private Sub C0Group_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles C0Group.Enter
        InEdit = True
    End Sub

    Private Sub C0Group_Leave(ByVal sender As Object, ByVal e As System.EventArgs) Handles C0Group.Leave
        InEdit = False
    End Sub

    Private Sub C1Group_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles C1Group.Enter
        InEdit = True
    End Sub

    Private Sub C1Group_Leave(ByVal sender As Object, ByVal e As System.EventArgs) Handles C1Group.Leave
        InEdit = False
    End Sub

    Private Sub C2Group_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles C2Group.Enter
        InEdit = True
    End Sub

    Private Sub C2Group_Leave(ByVal sender As Object, ByVal e As System.EventArgs) Handles C2Group.Leave
        InEdit = False
    End Sub

    Private Sub FreqGB_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles FreqGB.Enter
        InEdit = True
    End Sub
    Private Sub FreqGB_Leave(ByVal sender As Object, ByVal e As System.EventArgs) Handles FreqGB.Leave
        InEdit = False
    End Sub
    Private Sub MiscGB_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles MiscGB.Enter
        InEdit = True
    End Sub
    Private Sub MiscGB_Leave(ByVal sender As Object, ByVal e As System.EventArgs) Handles MiscGB.Leave
        InEdit = False
    End Sub
    Private Sub CommsGB_Enter(ByVal sender As Object, ByVal e As System.EventArgs) Handles CommsGB.Enter
        InEdit = True
    End Sub
    Private Sub CommsGB_Leave(ByVal sender As Object, ByVal e As System.EventArgs) Handles CommsGB.Leave
        InEdit = False
    End Sub
#End Region

#Region "Export"

    Private Sub ExportIt(ByRef ECurveNo As Integer)
        ' Export curve ECurveNo
        Dim GotMark As Integer
        If CurrentCurve(ECurveNo).NumPoints < 10 Then
            MsgBox("No Data to export")
        Else
            Try
                With SaveFileDialog1
                    .Title = "Select Export File"
                    .FileName = "Trace" & LTrim(ECurveNo.ToString)
                    .DefaultExt = "txt"
                    .Filter = "Text file (*.txt)|*.txt"
                    If .ShowDialog() = System.Windows.Forms.DialogResult.OK Then
                        FileOpen(1, System.IO.Path.Combine(Application.StartupPath, .FileName), OpenMode.Output)
                        Printcr(1, "VK3DIP GenSweep , Version " & MyVersion & ", Export of " & CurrentCurve(ECurveNo).MeName & " , " & Now.ToString)
                        Printcr(1, " Raw Frequency (Hz) , Raw Level (dB) , Display Frequency (Hz) , Display Level (dB) , Marker")
                        For j = 0 To CurrentCurve(ECurveNo).NumPoints - 1
                            GotMark = Index2Marker(j, ECurveNo)
                            If GotMark <> -1 Then
                                Printcr(1, CurrentCurve(ECurveNo).Values(j).RF.ToString & " , " & CurrentCurve(ECurveNo).Values(j).RL.ToString & " , " & CurrentCurve(ECurveNo).Values(j).DF.ToString & " , " & CurrentCurve(ECurveNo).Values(j).DL.ToString & " , " & GotMark.ToString)

                            Else
                                Printcr(1, CurrentCurve(ECurveNo).Values(j).RF.ToString & " , " & CurrentCurve(ECurveNo).Values(j).RL.ToString & " , " & CurrentCurve(ECurveNo).Values(j).DF.ToString & " , " & CurrentCurve(ECurveNo).Values(j).DL.ToString & " ,   ")

                            End If
                        Next
                        FileClose(1)
                    End If
                End With
            Catch ex As Exception
                MsgBox(ex.Message)
                FileClose(1)
            End Try
        End If

    End Sub
    Private Sub Export0B_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Export0B.Click
        ExportIt(0)
    End Sub
    Private Sub Export1B_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Export1B.Click
        ExportIt(1)
    End Sub

    Private Sub Export2B_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Export2B.Click
        ExportIt(2)
    End Sub

#End Region

    Private Sub A2DAvgNUD_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles A2DAvgNUD.ValueChanged
        A2DAvg = A2DAvgNUD.Value
    End Sub


    Private Sub AD9851CB_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles AD9851CB.CheckedChanged
        MessageBox.Show("Caution Do not set this to AD9851 unless you are sure you have one. Sending AD9851 control data to an AD9850 could permenantly damage it")
        AD9851Flg = AD9851CB.Checked
        If AD9851Flg Then
            MaxNUD.Value = 60000000
        Else
            MaxNUD.Value = 30000000
        End If
    End Sub

    Private Sub sweeper_ResizeEnd(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.ResizeEnd
        RecalcAll()
        setvert()
        CROScreen.Refresh()
    End Sub

    Private Sub SMB_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SMB.Click
        If SportsMode Then
            SMB.BackColor = SystemColors.Control
            SportsMode = False
        Else
            SMB.BackColor = Color.Yellow
            SportsMode = True
        End If

    End Sub
End Class
